﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="KeystrokeSettings.xaml.cs" company="arvystate.net">
//   arvystate.net
// </copyright>
// <summary>
//   Interaction logic for KeystrokeSettings.xaml
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace PresentationAid.Lib.GestureEngines.GestureSettings
{
    using System;
    using System.Linq;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Documents;
    using System.Windows.Input;

    using NLog;

    using PresentationAid.Lib.Enums;

    /// <summary>
    /// Interaction logic for KeystrokeSettings
    /// </summary>
    public partial class KeystrokeSettings
    {
        #region Static Fields

        /// <summary>
        /// The logger.
        /// </summary>
        private static readonly Logger Logger = LogManager.GetCurrentClassLogger();

        #endregion

        #region Fields

        /// <summary>
        /// The _keystroke engine.
        /// </summary>
        private readonly KeystrokeEngine keystrokeEngine;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="KeystrokeSettings"/> class.
        /// </summary>
        /// <param name="engine">
        /// The engine.
        /// </param>
        public KeystrokeSettings(GestureEngine engine)
            : base(engine)
        {
            Logger.Trace("Init: KeystrokeSettings");

            this.keystrokeEngine = (KeystrokeEngine)engine;

            this.InitializeComponent();
        }

        #endregion

        #region Public Methods and Operators

        /// <summary>
        /// Initializes setting boxes
        /// </summary>
        public override void CreateSettings()
        {
            // Create settings once.
            if (this.SettingGrid.RowDefinitions.Count > 1)
            {
                return;
            }

            for (int i = 1; i < Enum.GetNames(typeof(GestureCommand)).Length; i++)
            {
                this.SettingGrid.RowDefinitions.Add(new RowDefinition());

                // Create gesture combobox
                TextBlock textBlock = new TextBlock
                                          {
                                              Text = ((GestureCommand)i).ToString(), 
                                              Margin = new Thickness(15, 0, 15, 15), 
                                              FontSize = 16, 
                                              VerticalAlignment = VerticalAlignment.Center
                                          };

                Grid.SetRow(textBlock, i);

                this.SettingGrid.Children.Add(textBlock);

                // Create key combobox
                ComboBox comboBox = new ComboBox();

                // Handle control command
                ComboBoxItem item = new ComboBoxItem { Content = new TextBlock(new Bold(new Run("Take Control"))) };

                comboBox.Items.Add(item);

                for (int x = 0; x < Enum.GetNames(typeof(Key)).Length; x++)
                {
                    item = new ComboBoxItem { Content = ((Key)x).ToString() };

                    comboBox.Items.Add(item);
                }

                comboBox.Margin = new Thickness(0, 0, 15, 15);
                comboBox.SelectionChanged += this.OnComboBoxSelectionChanged;

                Grid.SetRow(comboBox, i);
                Grid.SetColumn(comboBox, 1);

                this.SettingGrid.Children.Add(comboBox);
            }
        }

        /// <summary>
        /// Adjusts combo boxes to current settings
        /// </summary>
        public override void LoadSettings()
        {
            Logger.Debug("Method: LoadSettings");

            // Can assume that the grid is setuped here
            for (int i = 1; i < Enum.GetNames(typeof(GestureCommand)).Length; i++)
            {
                // Get both ComboBoxes
                ComboBox keyMap =
                    (ComboBox)
                    this.SettingGrid.Children.Cast<UIElement>()
                        .First(e => (Grid.GetRow(e) == i) && (Grid.GetColumn(e) == 1));

                // Control Gesture
                if ((GestureCommand)i == this.keystrokeEngine.ControlGesture)
                {
                    keyMap.SelectedIndex = 0;
                }
                else
                {
                    // Loop through all keys
                    for (int x = 0; x < Enum.GetNames(typeof(Key)).Length; x++)
                    {
                        // Check for main and alternate key
                        if ((Key)x == this.keystrokeEngine.GetMapping((GestureCommand)i))
                        {
                            keyMap.SelectedIndex = x + 1;
                        }
                    }
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Handles key map changed and disables second combo box with control
        /// </summary>
        /// <param name="sender">
        /// Sender of the selection changed event
        /// </param>
        /// <param name="e">
        /// Event arguments
        /// </param>
        private void OnComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBox comboBox = (ComboBox)sender;

            // Disable alternate key box, if user selected take control action
            int row = Grid.GetRow(comboBox);

            Logger.Trace("EventDetail: ComboBoxSelectionChanged - Row: {0}", row);

            // Handle take control command
            if (comboBox.SelectedIndex == 0)
            {
                Logger.Trace("EventDetail: ComboBoxSelectionChanged - SetControlGesture");

                this.keystrokeEngine.ControlGesture = (GestureCommand)row;
            }
            else
            {
                Logger.Trace("EventDetail: ComboBoxSelectionChanged - SetMapping");

                if ((GestureCommand)row == this.keystrokeEngine.ControlGesture)
                {
                    this.keystrokeEngine.ControlGesture = GestureCommand.None;
                }

                this.keystrokeEngine.SetMapping((GestureCommand)row, (Key)(comboBox.SelectedIndex - 1));
            }

            // Handle control gesture removal
            if ((GestureCommand)row == this.keystrokeEngine.ControlGesture && comboBox.SelectedIndex == 0)
            {
                this.keystrokeEngine.ControlGesture = GestureCommand.None;
            }
        }

        #endregion
    }
}